@@ -17,12 +17,13 @@
 #   the converted revision to have a different identity than the
 #   source.
 
+from __future__ import annotations
+
 import os
 import re
 import time
 
 from mercurial.i18n import _
-from mercurial.pycompat import open
 from mercurial.node import (
     bin,
     hex,
@@ -696,7 +697,7 @@ class mercurial_source(common.converter_source):
         else:
             i = i or 0
             ma, r = self._changedfiles(parents[i], ctx)
-        ma, r = [[f for f in l if f not in self.ignored] for l in (ma, r)]
+        ma, r = ([f for f in l if f not in self.ignored] for l in (ma, r))
 
         if i == 0:
             self._changescache = (rev, (ma, r))
@@ -705,7 +706,7 @@ class mercurial_source(common.converter_source):
 
     def converted(self, rev, destrev):
         if self.convertfp is None:
-            self.convertfp = open(self.repo.vfs.join(b'shamap'), b'ab')
+            self.convertfp = open(self.repo.vfs.join(b'shamap'), 'ab')
         self.convertfp.write(util.tonativeeol(b'%s %s\n' % (destrev, rev)))
         self.convertfp.flush()
 
