@@ -5,6 +5,7 @@
 # This software may be used and distributed according to the terms of the
 # GNU General Public License version 2 or any later version.
 
+from __future__ import annotations
 
 import contextlib
 import errno
@@ -12,7 +13,10 @@ import hashlib
 import json
 import os
 import re
-import socket
+
+from typing import (
+    Optional,
+)
 
 from mercurial.i18n import _
 from mercurial.node import hex
@@ -41,11 +45,11 @@ _lfsre = re.compile(br'\A[a-f0-9]{64}\Z')
 
 
 class lfsvfs(vfsmod.vfs):
-    def join(self, path):
+    def join(self, path: Optional[bytes], *insidef: bytes) -> bytes:
         """split the path at first two characters, like: XX/XXXXX..."""
         if not _lfsre.match(path):
             raise error.ProgrammingError(b'unexpected lfs path: %s' % path)
-        return super(lfsvfs, self).join(path[0:2], path[2:])
+        return super().join(path[0:2], path[2:], *insidef)
 
     def walk(self, path=None, onerror=None):
         """Yield (dirpath, [], oids) tuple for blobs under path
@@ -76,7 +80,7 @@ class nullvfs(lfsvfs):
     def __init__(self):
         pass
 
-    def exists(self, oid):
+    def exists(self, path: Optional[bytes] = None) -> bool:
         return False
 
     def read(self, oid):
@@ -84,7 +88,7 @@ class nullvfs(lfsvfs):
         # self.vfs.  Raise the same error as a normal vfs when asked to read a
         # file that doesn't exist.  The only difference is the full file path
         # isn't available in the error.
-        raise IOError(
+        raise OSError(
             errno.ENOENT,
             pycompat.sysstr(b'%s: No such file or directory' % oid),
         )
@@ -92,15 +96,15 @@ class nullvfs(lfsvfs):
     def walk(self, path=None, onerror=None):
         return (b'', [], [])
 
-    def write(self, oid, data):
-        pass
+    def write(self, *args, **kwargs) -> int:
+        return 0
 
 
 class lfsuploadfile(httpconnectionmod.httpsendfile):
     """a file-like object that supports keepalive."""
 
     def __init__(self, ui, filename):
-        super(lfsuploadfile, self).__init__(ui, filename, b'rb')
+        super().__init__(ui, filename, b'rb')
         self.read = self._data.read
 
     def _makeprogress(self):
@@ -590,7 +594,7 @@ class _gitlfsremote:
                         self._basictransfer(obj, action, localstore)
                         yield 1, obj.get(b'oid')
                         break
-                    except socket.error as ex:
+                    except OSError as ex:
                         if retry > 0:
                             self.ui.note(
                                 _(b'lfs: failed: %r (remaining retry %d)\n')
