@@ -5,19 +5,23 @@
 # This software may be used and distributed according to the terms of the
 # GNU General Public License version 2 or any later version.
 
-from __future__ import absolute_import
+from __future__ import annotations
 
 import gzip
 import os
 import struct
 import tarfile
 import time
+import typing
 import zipfile
 import zlib
 
+from typing import (
+    Optional,
+)
+
 from .i18n import _
 from .node import nullrev
-from .pycompat import open
 
 from . import (
     error,
@@ -31,6 +35,11 @@ from . import (
 
 from .utils import stringutil
 
+if typing.TYPE_CHECKING:
+    from . import (
+        localrepo,
+    )
+
 stringio = util.stringio
 
 # from unzip source code:
@@ -46,7 +55,7 @@ def tidyprefix(dest, kind, prefix):
         prefix = util.normpath(prefix)
     else:
         if not isinstance(dest, bytes):
-            raise ValueError(b'dest must be string if no prefix')
+            raise ValueError('dest must be bytes if no prefix')
         prefix = os.path.basename(dest)
         lower = prefix.lower()
         for sfx in exts.get(kind, []):
@@ -76,7 +85,7 @@ exts = {
 
 
 def guesskind(dest):
-    for kind, extensions in pycompat.iteritems(exts):
+    for kind, extensions in exts.items():
         if any(dest.endswith(ext) for ext in extensions):
             return kind
     return None
@@ -133,77 +142,40 @@ def buildmetadata(ctx):
     return out.getvalue()
 
 
-class tarit(object):
+class tarit:
     """write archive to tar file or stream.  can write uncompressed,
     or compress with gzip or bzip2."""
 
-    if pycompat.ispy3:
-        GzipFileWithTime = gzip.GzipFile  # camelcase-required
-    else:
-
-        class GzipFileWithTime(gzip.GzipFile):
-            def __init__(self, *args, **kw):
-                timestamp = None
-                if 'mtime' in kw:
-                    timestamp = kw.pop('mtime')
-                if timestamp is None:
-                    self.timestamp = time.time()
-                else:
-                    self.timestamp = timestamp
-                gzip.GzipFile.__init__(self, *args, **kw)
-
-            def _write_gzip_header(self):
-                self.fileobj.write(b'\037\213')  # magic header
-                self.fileobj.write(b'\010')  # compression method
-                fname = self.name
-                if fname and fname.endswith(b'.gz'):
-                    fname = fname[:-3]
-                flags = 0
-                if fname:
-                    flags = gzip.FNAME  # pytype: disable=module-attr
-                self.fileobj.write(pycompat.bytechr(flags))
-                gzip.write32u(  # pytype: disable=module-attr
-                    self.fileobj, int(self.timestamp)
-                )
-                self.fileobj.write(b'\002')
-                self.fileobj.write(b'\377')
-                if fname:
-                    self.fileobj.write(fname + b'\000')
-
-    def __init__(self, dest, mtime, kind=b''):
+    def __init__(self, dest, mtime, kind: str = ''):
         self.mtime = mtime
         self.fileobj = None
 
-        def taropen(mode, name=b'', fileobj=None):
-            if kind == b'gz':
+        def taropen(
+            mode: str, name: bytes = b'', fileobj=None
+        ) -> tarfile.TarFile:
+            if kind == 'gz':
                 mode = mode[0:1]
                 if not fileobj:
-                    fileobj = open(name, mode + b'b')
-                gzfileobj = self.GzipFileWithTime(
+                    fileobj = open(name, mode + 'b')
+                gzfileobj = gzip.GzipFile(
                     name,
-                    pycompat.sysstr(mode + b'b'),
+                    mode + 'b',
                     zlib.Z_BEST_COMPRESSION,
                     fileobj,
                     mtime=mtime,
                 )
                 self.fileobj = gzfileobj
-                return (
-                    tarfile.TarFile.taropen(  # pytype: disable=attribute-error
-                        name, pycompat.sysstr(mode), gzfileobj
-                    )
-                )
+                return tarfile.TarFile.taropen(name, "w", gzfileobj)
             else:
                 try:
-                    return tarfile.open(
-                        name, pycompat.sysstr(mode + kind), fileobj
-                    )
+                    return tarfile.open(name, mode + kind, fileobj)
                 except tarfile.CompressionError as e:
                     raise error.Abort(stringutil.forcebytestr(e))
 
         if isinstance(dest, bytes):
-            self.z = taropen(b'w:', name=dest)
+            self.z = taropen('w:', name=dest)
         else:
-            self.z = taropen(b'w|', fileobj=dest)
+            self.z = taropen('w|', fileobj=dest)
 
     def addfile(self, name, mode, islink, data):
         name = pycompat.fsdecode(name)
@@ -227,7 +199,7 @@ class tarit(object):
             self.fileobj.close()
 
 
-class zipit(object):
+class zipit:
     """write archive to zip file or stream.  can write uncompressed,
     or compressed with deflate."""
 
@@ -274,7 +246,7 @@ class zipit(object):
         self.z.close()
 
 
-class fileit(object):
+class fileit:
     '''write archive as files in directory.'''
 
     def __init__(self, name, mtime):
@@ -301,29 +273,30 @@ class fileit(object):
 archivers = {
     b'files': fileit,
     b'tar': tarit,
-    b'tbz2': lambda name, mtime: tarit(name, mtime, b'bz2'),
-    b'tgz': lambda name, mtime: tarit(name, mtime, b'gz'),
-    b'txz': lambda name, mtime: tarit(name, mtime, b'xz'),
+    b'tbz2': lambda name, mtime: tarit(name, mtime, 'bz2'),
+    b'tgz': lambda name, mtime: tarit(name, mtime, 'gz'),
+    b'txz': lambda name, mtime: tarit(name, mtime, 'xz'),
     b'uzip': lambda name, mtime: zipit(name, mtime, False),
     b'zip': zipit,
 }
 
 
 def archive(
-    repo,
-    dest,
+    repo: localrepo.localrepository,
+    dest,  # TODO: should be bytes, but could be Callable
     node,
-    kind,
-    decode=True,
+    kind: bytes,
+    decode: bool = True,
     match=None,
-    prefix=b'',
-    mtime=None,
-    subrepos=False,
-):
+    prefix: bytes = b'',
+    mtime: Optional[float] = None,
+    subrepos: bool = False,
+) -> int:
     """create archive of repo as it was at node.
 
-    dest can be name of directory, name of archive file, or file
-    object to write archive to.
+    dest can be name of directory, name of archive file, a callable, or file
+    object to write archive to. If it is a callable, it will called to open
+    the actual file object before the first archive member is written.
 
     kind is type of archive to create.
 
@@ -339,8 +312,8 @@ def archive(
     subrepos tells whether to include subrepos.
     """
 
-    if kind == b'txz' and not pycompat.ispy3:
-        raise error.Abort(_(b'xz compression is only available in Python 3'))
+    if kind not in archivers:
+        raise error.Abort(_(b"unknown archive type '%s'") % kind)
 
     if kind == b'files':
         if prefix:
@@ -348,26 +321,45 @@ def archive(
     else:
         prefix = tidyprefix(dest, kind, prefix)
 
+    archiver = None
+    ctx = repo[node]
+
+    def opencallback():
+        """Return the archiver instance, creating it if necessary.
+
+        This function is called when the first actual entry is created.
+        It may be called multiple times from different layers.
+        When serving the archive via hgweb, no errors should happen after
+        this point.
+        """
+        nonlocal archiver
+        if archiver is None:
+            if callable(dest):
+                output = dest()
+            else:
+                output = dest
+            archiver = archivers[kind](output, mtime or ctx.date()[0])
+            assert archiver is not None
+
+            if repo.ui.configbool(b"ui", b"archivemeta"):
+                metaname = b'.hg_archival.txt'
+                if match(metaname):
+                    write(metaname, 0o644, False, lambda: buildmetadata(ctx))
+        return archiver
+
     def write(name, mode, islink, getdata):
+        if archiver is None:
+            opencallback()
+        assert archiver is not None, "archive should be opened by now"
+
         data = getdata()
         if decode:
             data = repo.wwritedata(name, data)
         archiver.addfile(prefix + name, mode, islink, data)
 
-    if kind not in archivers:
-        raise error.Abort(_(b"unknown archive type '%s'") % kind)
-
-    ctx = repo[node]
-    archiver = archivers[kind](dest, mtime or ctx.date()[0])
-
     if not match:
         match = scmutil.matchall(repo)
 
-    if repo.ui.configbool(b"ui", b"archivemeta"):
-        name = b'.hg_archival.txt'
-        if match(name):
-            write(name, 0o644, False, lambda: buildmetadata(ctx))
-
     files = list(ctx.manifest().walk(match))
     total = len(files)
     if total:
@@ -390,10 +382,11 @@ def archive(
             sub = ctx.workingsub(subpath)
             submatch = matchmod.subdirmatcher(subpath, match)
             subprefix = prefix + subpath + b'/'
-            total += sub.archive(archiver, subprefix, submatch, decode)
+            total += sub.archive(opencallback, subprefix, submatch, decode)
 
     if total == 0:
         raise error.Abort(_(b'no files match the archive pattern'))
 
+    assert archiver is not None, "archive should have been opened before"
     archiver.done()
     return total
