@@ -90,6 +90,7 @@ struct indexObjectStruct {
 	int ntlookups;          /* # lookups */
 	int ntmisses;           /* # lookups that miss the cache */
 	int inlined;
+	int uses_generaldelta; /* whether this index uses generaldelta */
 	long entry_size; /* size of index headers. Differs in v1 v.s. v2 format
 	                  */
 	long rust_ext_compat; /* compatibility with being used in rust
@@ -1237,22 +1238,36 @@ release:
 
 static PyObject *index_headrevs(indexObject *self, PyObject *args)
 {
-	Py_ssize_t i, j, len;
+	Py_ssize_t i, j, len, stop_rev;
 	char *nothead = NULL;
 	PyObject *heads = NULL;
 	PyObject *filter = NULL;
 	PyObject *filteredrevs = Py_None;
+	PyObject *stop_rev_obj = Py_None;
 
-	if (!PyArg_ParseTuple(args, "|O", &filteredrevs)) {
+	if (!PyArg_ParseTuple(args, "|OO", &filteredrevs, &stop_rev_obj)) {
 		return NULL;
 	}
 
-	if (self->headrevs && filteredrevs == self->filteredrevs)
+	len = index_length(self);
+	if (stop_rev_obj == Py_None) {
+		stop_rev = len;
+	} else {
+		stop_rev = PyLong_AsSsize_t(stop_rev_obj);
+		if (stop_rev == -1 && PyErr_Occurred() != NULL) {
+			return NULL;
+		}
+	}
+
+	if (self->headrevs && filteredrevs == self->filteredrevs &&
+	    stop_rev == len)
 		return list_copy(self->headrevs);
 
-	Py_DECREF(self->filteredrevs);
-	self->filteredrevs = filteredrevs;
-	Py_INCREF(filteredrevs);
+	if (stop_rev == len) {
+		Py_DECREF(self->filteredrevs);
+		self->filteredrevs = filteredrevs;
+		Py_INCREF(filteredrevs);
+	}
 
 	if (filteredrevs != Py_None) {
 		filter = PyObject_GetAttrString(filteredrevs, "__contains__");
@@ -1264,11 +1279,10 @@ static PyObject *index_headrevs(indexObject *self, PyObject *args)
 		}
 	}
 
-	len = index_length(self);
 	heads = PyList_New(0);
 	if (heads == NULL)
 		goto bail;
-	if (len == 0) {
+	if (stop_rev == 0) {
 		if (pylist_append_owned(heads, PyLong_FromLong(-1)) == -1) {
 			Py_XDECREF(nullid);
 			goto bail;
@@ -1276,13 +1290,13 @@ static PyObject *index_headrevs(indexObject *self, PyObject *args)
 		goto done;
 	}
 
-	nothead = calloc(len, 1);
+	nothead = calloc(stop_rev, 1);
 	if (nothead == NULL) {
 		PyErr_NoMemory();
 		goto bail;
 	}
 
-	for (i = len - 1; i >= 0; i--) {
+	for (i = stop_rev - 1; i >= 0; i--) {
 		int isfiltered;
 		int parents[2];
 
@@ -1304,7 +1318,7 @@ static PyObject *index_headrevs(indexObject *self, PyObject *args)
 			}
 		}
 
-		if (index_get_parents(self, i, parents, (int)len - 1) < 0)
+		if (index_get_parents(self, i, parents, (int)stop_rev - 1) < 0)
 			goto bail;
 		for (j = 0; j < 2; j++) {
 			if (parents[j] >= 0)
@@ -1312,7 +1326,7 @@ static PyObject *index_headrevs(indexObject *self, PyObject *args)
 		}
 	}
 
-	for (i = 0; i < len; i++) {
+	for (i = 0; i < stop_rev; i++) {
 		if (nothead[i])
 			continue;
 		if (pylist_append_owned(heads, PyLong_FromSsize_t(i)) == -1) {
@@ -1321,10 +1335,14 @@ static PyObject *index_headrevs(indexObject *self, PyObject *args)
 	}
 
 done:
-	self->headrevs = heads;
 	Py_XDECREF(filter);
 	free(nothead);
-	return list_copy(self->headrevs);
+	if (stop_rev == len) {
+		self->headrevs = heads;
+		return list_copy(self->headrevs);
+	} else {
+		return heads;
+	}
 bail:
 	Py_XDECREF(filter);
 	Py_XDECREF(heads);
@@ -1707,14 +1725,14 @@ bail:
 
 static PyObject *index_deltachain(indexObject *self, PyObject *args)
 {
-	int rev, generaldelta;
+	int rev;
 	PyObject *stoparg;
 	int stoprev, iterrev, baserev = -1;
 	int stopped;
 	PyObject *chain = NULL, *result = NULL;
 	const Py_ssize_t length = index_length(self);
 
-	if (!PyArg_ParseTuple(args, "iOi", &rev, &stoparg, &generaldelta)) {
+	if (!PyArg_ParseTuple(args, "iO", &rev, &stoparg)) {
 		return NULL;
 	}
 
@@ -1757,7 +1775,7 @@ static PyObject *index_deltachain(indexObject *self, PyObject *args)
 			goto bail;
 		}
 
-		if (generaldelta) {
+		if (self->uses_generaldelta) {
 			iterrev = baserev;
 		} else {
 			iterrev--;
@@ -3189,10 +3207,11 @@ static Py_ssize_t inline_scan(indexObject *self, const char **offsets)
 
 static int index_init(indexObject *self, PyObject *args, PyObject *kwargs)
 {
-	PyObject *data_obj, *inlined_obj;
+	PyObject *data_obj, *inlined_obj, *generaldelta_obj;
 	Py_ssize_t size;
 
-	static char *kwlist[] = {"data", "inlined", "format", NULL};
+	static char *kwlist[] = {"data", "inlined", "uses_generaldelta",
+	                         "format", NULL};
 
 	/* Initialize before argument-checking to avoid index_dealloc() crash.
 	 */
@@ -3208,12 +3227,13 @@ static int index_init(indexObject *self, PyObject *args, PyObject *kwargs)
 	self->offsets = NULL;
 	self->nodelen = 20;
 	self->nullentry = NULL;
+	self->uses_generaldelta = 0;
 	self->rust_ext_compat = 0;
 	self->format_version = format_v1;
 
-	if (!PyArg_ParseTupleAndKeywords(args, kwargs, "OO|l", kwlist,
-	                                 &data_obj, &inlined_obj,
-	                                 &(self->format_version)))
+	if (!PyArg_ParseTupleAndKeywords(
+	        args, kwargs, "OOO|l", kwlist, &data_obj, &inlined_obj,
+	        &generaldelta_obj, &(self->format_version)))
 		return -1;
 	if (!PyObject_CheckBuffer(data_obj)) {
 		PyErr_SetString(PyExc_TypeError,
@@ -3246,6 +3266,8 @@ static int index_init(indexObject *self, PyObject *args, PyObject *kwargs)
 	size = self->buf.len;
 
 	self->inlined = inlined_obj && PyObject_IsTrue(inlined_obj);
+	self->uses_generaldelta =
+	    generaldelta_obj && PyObject_IsTrue(generaldelta_obj);
 	self->data = data_obj;
 
 	self->ntlookups = self->ntmisses = 0;
@@ -3352,9 +3374,7 @@ static PyMethodDef index_methods[] = {
     {"replace_sidedata_info", (PyCFunction)index_replace_sidedata_info,
      METH_VARARGS, "replace an existing index entry with a new value"},
     {"headrevs", (PyCFunction)index_headrevs, METH_VARARGS,
-     "get head revisions"}, /* Can do filtering since 3.2 */
-    {"headrevsfiltered", (PyCFunction)index_headrevs, METH_VARARGS,
-     "get filtered head revisions"}, /* Can always do filtering */
+     "get head revisions"},
     {"issnapshot", (PyCFunction)index_issnapshot, METH_O,
      "True if the object is a snapshot"},
     {"findsnapshots", (PyCFunction)index_findsnapshots, METH_VARARGS,
