@@ -5,6 +5,7 @@
 # This software may be used and distributed according to the terms of the
 # GNU General Public License version 2 or any later version.
 
+from __future__ import annotations
 
 import gc
 import os
@@ -720,10 +721,10 @@ class unixforkingservice:
 
         class unixcmdserverrepo(repo.__class__):
             def close(self):
-                super(unixcmdserverrepo, self).close()
+                super().close()
                 try:
                     self._cmdserveripc.send(self.root)
-                except socket.error:
+                except OSError:
                     self.ui.log(
                         b'cmdserver', b'failed to send repo root to master\n'
                     )
