@@ -5,6 +5,7 @@
 # This software may be used and distributed according to the terms of the
 # GNU General Public License version 2 or any later version.
 
+from __future__ import annotations
 
 import errno
 import os
@@ -51,8 +52,7 @@ class config:
         return self._data.get(section, {})
 
     def __iter__(self):
-        for d in self.sections():
-            yield d
+        yield from self.sections()
 
     def update(self, src):
         current_level = self._current_source_level
@@ -195,7 +195,7 @@ class config:
                 expanded = util.expandpath(m.group(1))
                 try:
                     include(expanded, remap=remap, sections=sections)
-                except IOError as inst:
+                except OSError as inst:
                     if inst.errno != errno.ENOENT:
                         raise error.ConfigError(
                             _(b"cannot include %s (%s)")
