@@ -6,6 +6,7 @@
 # This software may be used and distributed according to the terms of the
 # GNU General Public License version 2 or any later version.
 
+from __future__ import annotations
 
 import copy
 import difflib
@@ -600,7 +601,7 @@ def changesetentry(web, ctx):
         diffsummary=lambda context, mapping: diffsummary(diffstatsgen),
         diffstat=diffstats,
         archives=web.archivelist(ctx.hex()),
-        **pycompat.strkwargs(commonentry(web.repo, ctx))
+        **pycompat.strkwargs(commonentry(web.repo, ctx)),
     )
 
 
