@@ -1,7 +1,6 @@
-from __future__ import absolute_import
+from __future__ import annotations
 
 import collections
-import errno
 import shutil
 import struct
 import weakref
@@ -15,7 +14,6 @@ from .node import (
 from . import (
     error,
     filemerge,
-    pycompat,
     util,
 )
 from .utils import hashutil
@@ -103,7 +101,7 @@ CHANGE_REMOVED = b'removed'
 CHANGE_MODIFIED = b'modified'
 
 
-class MergeAction(object):
+class MergeAction:
     """represent an "action" merge need to take for a given file
 
     Attributes:
@@ -163,7 +161,7 @@ ACTION_REMOVE = MergeAction(b'r', narrow_safe=True, changes=CHANGE_REMOVED)
 ACTION_ADD = MergeAction(b'a', narrow_safe=True, changes=CHANGE_ADDED)
 ACTION_GET = MergeAction(b'g', narrow_safe=True, changes=CHANGE_MODIFIED)
 ACTION_PATH_CONFLICT = MergeAction(b'p')
-ACTION_PATH_CONFLICT_RESOLVE = MergeAction('pr')
+ACTION_PATH_CONFLICT_RESOLVE = MergeAction(b'pr')
 ACTION_ADD_MODIFIED = MergeAction(
     b'am', narrow_safe=True, changes=CHANGE_ADDED
 )  # not 100% about the changes value here
@@ -197,7 +195,7 @@ CONVERT_MERGE_ACTIONS = (
 )
 
 
-class _mergestate_base(object):
+class _mergestate_base:
     """track 3-way merge state of individual files
 
     The merge state is stored on disk when needed. Two files are used: one with
@@ -365,7 +363,7 @@ class _mergestate_base(object):
     def unresolved(self):
         """Obtain the paths of unresolved files."""
 
-        for f, entry in pycompat.iteritems(self._state):
+        for f, entry in self._state.items():
             if entry[0] in (
                 MERGE_RECORD_UNRESOLVED,
                 MERGE_RECORD_UNRESOLVED_PATH,
@@ -469,7 +467,7 @@ class _mergestate_base(object):
         """return counts for updated, merged and removed files in this
         session"""
         updated, merged, removed = 0, 0, 0
-        for r, action in pycompat.itervalues(self._results):
+        for r, action in self._results.values():
             if r is None:
                 updated += 1
             elif r == 0:
@@ -492,14 +490,13 @@ class _mergestate_base(object):
             ACTION_ADD_MODIFIED: [],
             ACTION_GET: [],
         }
-        for f, (r, action) in pycompat.iteritems(self._results):
+        for f, (r, action) in self._results.items():
             if action is not None:
                 actions[action].append((f, None, b"merge result"))
         return actions
 
 
 class mergestate(_mergestate_base):
-
     statepathv1 = b'merge/state'
     statepathv2 = b'merge/state2'
 
@@ -632,9 +629,8 @@ class mergestate(_mergestate_base):
                 else:
                     records.append((RECORD_MERGED, l[:-1]))
             f.close()
-        except IOError as err:
-            if err.errno != errno.ENOENT:
-                raise
+        except FileNotFoundError:
+            pass
         return records
 
     def _readrecordsv2(self):
@@ -672,9 +668,8 @@ class mergestate(_mergestate_base):
                     rtype, record = record[0:1], record[1:]
                 records.append((rtype, record))
             f.close()
-        except IOError as err:
-            if err.errno != errno.ENOENT:
-                raise
+        except FileNotFoundError:
+            pass
         return records
 
     def commit(self):
@@ -692,7 +687,7 @@ class mergestate(_mergestate_base):
         # the type of state that is stored, and capital-letter records are used
         # to prevent older versions of Mercurial that do not support the feature
         # from loading them.
-        for filename, v in pycompat.iteritems(self._state):
+        for filename, v in self._state.items():
             if v[0] in (
                 MERGE_RECORD_UNRESOLVED_PATH,
                 MERGE_RECORD_RESOLVED_PATH,
@@ -716,9 +711,9 @@ class mergestate(_mergestate_base):
             else:
                 # Normal files.  These are stored in 'F' records.
                 records.append((RECORD_MERGED, b'\0'.join([filename] + v)))
-        for filename, extras in sorted(pycompat.iteritems(self._stateextras)):
+        for filename, extras in sorted(self._stateextras.items()):
             rawextras = b'\0'.join(
-                b'%s\0%s' % (k, v) for k, v in pycompat.iteritems(extras)
+                b'%s\0%s' % (k, v) for k, v in extras.items()
             )
             records.append(
                 (RECORD_FILE_VALUES, b'%s\0%s' % (filename, rawextras))
@@ -773,7 +768,7 @@ class mergestate(_mergestate_base):
 
 class memmergestate(_mergestate_base):
     def __init__(self, repo):
-        super(memmergestate, self).__init__(repo)
+        super().__init__(repo)
         self._backups = {}
 
     def _make_backup(self, fctx, localkey):
@@ -785,45 +780,46 @@ class memmergestate(_mergestate_base):
 
 def recordupdates(repo, actions, branchmerge, getfiledata):
     """record merge actions to the dirstate"""
+    dirstate = repo.dirstate
+    update_file = dirstate.update_file
+
     # remove (must come first)
     for f, args, msg in actions.get(ACTION_REMOVE, []):
         if branchmerge:
-            repo.dirstate.update_file(f, p1_tracked=True, wc_tracked=False)
+            update_file(f, p1_tracked=True, wc_tracked=False)
         else:
-            repo.dirstate.update_file(f, p1_tracked=False, wc_tracked=False)
+            update_file(f, p1_tracked=False, wc_tracked=False)
 
     # forget (must come first)
     for f, args, msg in actions.get(ACTION_FORGET, []):
-        repo.dirstate.update_file(f, p1_tracked=False, wc_tracked=False)
+        update_file(f, p1_tracked=False, wc_tracked=False)
 
     # resolve path conflicts
     for f, args, msg in actions.get(ACTION_PATH_CONFLICT_RESOLVE, []):
         (f0, origf0) = args
-        repo.dirstate.update_file(f, p1_tracked=False, wc_tracked=True)
-        repo.dirstate.copy(origf0, f)
+        update_file(f, p1_tracked=False, wc_tracked=True)
+        dirstate.copy(origf0, f)
         if f0 == origf0:
-            repo.dirstate.update_file(f0, p1_tracked=True, wc_tracked=False)
+            update_file(f0, p1_tracked=True, wc_tracked=False)
         else:
-            repo.dirstate.update_file(f0, p1_tracked=False, wc_tracked=False)
+            update_file(f0, p1_tracked=False, wc_tracked=False)
 
     # re-add
     for f, args, msg in actions.get(ACTION_ADD, []):
-        repo.dirstate.update_file(f, p1_tracked=False, wc_tracked=True)
+        update_file(f, p1_tracked=False, wc_tracked=True)
 
     # re-add/mark as modified
     for f, args, msg in actions.get(ACTION_ADD_MODIFIED, []):
         if branchmerge:
-            repo.dirstate.update_file(
+            update_file(
                 f, p1_tracked=True, wc_tracked=True, possibly_dirty=True
             )
         else:
-            repo.dirstate.update_file(f, p1_tracked=False, wc_tracked=True)
+            update_file(f, p1_tracked=False, wc_tracked=True)
 
     # exec change
     for f, args, msg in actions.get(ACTION_EXEC, []):
-        repo.dirstate.update_file(
-            f, p1_tracked=True, wc_tracked=True, possibly_dirty=True
-        )
+        update_file(f, p1_tracked=True, wc_tracked=True, possibly_dirty=True)
 
     # keep
     for f, args, msg in actions.get(ACTION_KEEP, []):
@@ -841,9 +837,9 @@ def recordupdates(repo, actions, branchmerge, getfiledata):
     for f, args, msg in actions.get(ACTION_GET, []):
         if branchmerge:
             # tracked in p1 can be True also but update_file should not care
-            old_entry = repo.dirstate.get_entry(f)
+            old_entry = dirstate.get_entry(f)
             p1_tracked = old_entry.any_tracked and not old_entry.added
-            repo.dirstate.update_file(
+            update_file(
                 f,
                 p1_tracked=p1_tracked,
                 wc_tracked=True,
@@ -851,7 +847,7 @@ def recordupdates(repo, actions, branchmerge, getfiledata):
             )
         else:
             parentfiledata = getfiledata[f] if getfiledata else None
-            repo.dirstate.update_file(
+            update_file(
                 f,
                 p1_tracked=True,
                 wc_tracked=True,
@@ -865,7 +861,7 @@ def recordupdates(repo, actions, branchmerge, getfiledata):
             # We've done a branch merge, mark this file as merged
             # so that we properly record the merger later
             p1_tracked = f1 == f
-            repo.dirstate.update_file(
+            update_file(
                 f,
                 p1_tracked=p1_tracked,
                 wc_tracked=True,
@@ -873,13 +869,11 @@ def recordupdates(repo, actions, branchmerge, getfiledata):
             )
             if f1 != f2:  # copy/rename
                 if move:
-                    repo.dirstate.update_file(
-                        f1, p1_tracked=True, wc_tracked=False
-                    )
+                    update_file(f1, p1_tracked=True, wc_tracked=False)
                 if f1 != f:
-                    repo.dirstate.copy(f1, f)
+                    dirstate.copy(f1, f)
                 else:
-                    repo.dirstate.copy(f2, f)
+                    dirstate.copy(f2, f)
         else:
             # We've update-merged a locally modified file, so
             # we set the dirstate to emulate a normal checkout
@@ -887,30 +881,28 @@ def recordupdates(repo, actions, branchmerge, getfiledata):
             # merge will appear as a normal local file
             # modification.
             if f2 == f:  # file not locally copied/moved
-                repo.dirstate.update_file(
+                update_file(
                     f, p1_tracked=True, wc_tracked=True, possibly_dirty=True
                 )
             if move:
-                repo.dirstate.update_file(
-                    f1, p1_tracked=False, wc_tracked=False
-                )
+                update_file(f1, p1_tracked=False, wc_tracked=False)
 
     # directory rename, move local
     for f, args, msg in actions.get(ACTION_DIR_RENAME_MOVE_LOCAL, []):
         f0, flag = args
         if branchmerge:
-            repo.dirstate.update_file(f, p1_tracked=False, wc_tracked=True)
-            repo.dirstate.update_file(f0, p1_tracked=True, wc_tracked=False)
-            repo.dirstate.copy(f0, f)
+            update_file(f, p1_tracked=False, wc_tracked=True)
+            update_file(f0, p1_tracked=True, wc_tracked=False)
+            dirstate.copy(f0, f)
         else:
-            repo.dirstate.update_file(f, p1_tracked=True, wc_tracked=True)
-            repo.dirstate.update_file(f0, p1_tracked=False, wc_tracked=False)
+            update_file(f, p1_tracked=True, wc_tracked=True)
+            update_file(f0, p1_tracked=False, wc_tracked=False)
 
     # directory rename, get
     for f, args, msg in actions.get(ACTION_LOCAL_DIR_RENAME_GET, []):
         f0, flag = args
         if branchmerge:
-            repo.dirstate.update_file(f, p1_tracked=False, wc_tracked=True)
-            repo.dirstate.copy(f0, f)
+            update_file(f, p1_tracked=False, wc_tracked=True)
+            dirstate.copy(f0, f)
         else:
-            repo.dirstate.update_file(f, p1_tracked=True, wc_tracked=True)
+            update_file(f, p1_tracked=True, wc_tracked=True)
