@@ -100,6 +100,7 @@ Note: old client behave as a publishing server with draft only content
 
 """
 
+from __future__ import annotations
 
 import heapq
 import struct
@@ -109,12 +110,14 @@ import weakref
 from typing import (
     Any,
     Callable,
+    Collection,
     Dict,
     Iterable,
     List,
     Optional,
     Set,
     Tuple,
+    overload,
 )
 
 from .i18n import _
@@ -127,7 +130,6 @@ from .node import (
 )
 from . import (
     error,
-    pycompat,
     requirements,
     smartset,
     txnutil,
@@ -138,6 +140,9 @@ Phaseroots = Dict[int, Set[int]]
 PhaseSets = Dict[int, Set[int]]
 
 if typing.TYPE_CHECKING:
+    from typing_extensions import (
+        Literal,  # py3.8+
+    )
     from . import (
         localrepo,
         ui as uimod,
@@ -168,10 +173,12 @@ phasenames = dict(enumerate(cmdphasenames))
 phasenames[archived] = b'archived'
 phasenames[internal] = b'internal'
 # map phase name to phase number
-phasenumber = {name: phase for phase, name in phasenames.items()}
+phasenumber: dict[bytes, int] = {
+    name: phase for phase, name in phasenames.items()
+}
 # like phasenumber, but also include maps for the numeric and binary
 # phase number to the phase number
-phasenumber2 = phasenumber.copy()
+phasenumber2: dict[bytes | int, int] = phasenumber.copy()
 phasenumber2.update({phase: phase for phase in phasenames})
 phasenumber2.update({b'%i' % phase: phase for phase in phasenames})
 # record phase property
@@ -185,19 +192,19 @@ all_internal_phases = tuple(p for p in allphases if p & internal)
 no_bundle_phases = all_internal_phases
 
 
-def supportinternal(repo: "localrepo.localrepository") -> bool:
+def supportinternal(repo: localrepo.localrepository) -> bool:
     """True if the internal phase can be used on a repository"""
     return requirements.INTERNAL_PHASE_REQUIREMENT in repo.requirements
 
 
-def supportarchived(repo: "localrepo.localrepository") -> bool:
+def supportarchived(repo: localrepo.localrepository) -> bool:
     """True if the archived phase can be used on a repository"""
     return requirements.ARCHIVED_PHASE_REQUIREMENT in repo.requirements
 
 
 def _readroots(
-    repo: "localrepo.localrepository",
-    phasedefaults: Optional["Phasedefaults"] = None,
+    repo: localrepo.localrepository,
+    phasedefaults: Optional[Phasedefaults] = None,
 ) -> Tuple[Phaseroots, bool]:
     """Read phase roots from disk
 
@@ -211,7 +218,7 @@ def _readroots(
     """
     repo = repo.unfiltered()
     dirty = False
-    roots = {i: set() for i in allphases}
+    roots: Phaseroots = {i: set() for i in allphases}
     to_rev = repo.changelog.index.get_rev
     unknown_msg = b'removing unknown node %s from %i-phase boundary\n'
     try:
@@ -375,11 +382,31 @@ INCREMENTAL_PHASE_SETS_UPDATE_MAX_UPDATE = 100
 
 
 class phasecache:
+    if typing.TYPE_CHECKING:
+
+        @overload
+        def __init__(
+            self,
+            repo: Any,
+            phasedefaults: Any,
+            _load: Literal[False],
+        ) -> None:
+            pass
+
+        @overload
+        def __init__(
+            self,
+            repo: localrepo.localrepository,
+            phasedefaults: Optional[Phasedefaults],
+            _load: bool = True,
+        ) -> None:
+            pass
+
     def __init__(
         self,
-        repo: "localrepo.localrepository",
-        phasedefaults: Optional["Phasedefaults"],
-        _load: bool = True,
+        repo,
+        phasedefaults,
+        _load=True,
     ):
         if _load:
             # Cheap trick to allow shallow-copy without copy module
@@ -387,18 +414,16 @@ class phasecache:
             self._phaseroots: Phaseroots = loaded[0]
             self.dirty: bool = loaded[1]
             self._loadedrevslen = 0
-            self._phasesets: PhaseSets = None
+            self._phasesets: Optional[PhaseSets] = None
 
-    def hasnonpublicphases(self, repo: "localrepo.localrepository") -> bool:
+    def hasnonpublicphases(self, repo: localrepo.localrepository) -> bool:
         """detect if there are revisions with non-public phase"""
         # XXX deprecate the unused repo argument
         return any(
             revs for phase, revs in self._phaseroots.items() if phase != public
         )
 
-    def nonpublicphaseroots(
-        self, repo: "localrepo.localrepository"
-    ) -> Set[int]:
+    def nonpublicphaseroots(self, repo: localrepo.localrepository) -> Set[int]:
         """returns the roots of all non-public phases
 
         The roots are not minimized, so if the secret revisions are
@@ -414,9 +439,30 @@ class phasecache:
             ]
         )
 
+    def get_raw_set(
+        self,
+        repo: localrepo.localrepository,
+        phase: int,
+    ) -> Set[int]:
+        """return the set of revision in that phase
+
+        The returned set is not filtered and might contains revision filtered
+        for the passed repoview.
+
+        The returned set might be the internal one and MUST NOT be mutated to
+        avoid side effect.
+        """
+        if phase == public:
+            raise error.ProgrammingError("cannot get_set for public phase")
+        self._ensure_phase_sets(repo.unfiltered())
+        revs = self._phasesets.get(phase)
+        if revs is None:
+            return set()
+        return revs
+
     def getrevset(
         self,
-        repo: "localrepo.localrepository",
+        repo: localrepo.localrepository,
         phases: Iterable[int],
         subset: Optional[Any] = None,
     ) -> Any:
@@ -516,7 +562,7 @@ class phasecache:
                 self._phasesets[phase] = ps
         self._loadedrevslen = len(cl)
 
-    def _ensure_phase_sets(self, repo: "localrepo.localrepository") -> None:
+    def _ensure_phase_sets(self, repo: localrepo.localrepository) -> None:
         """ensure phase information is loaded in the object"""
         assert repo.filtername is None
         update = -1
@@ -588,7 +634,7 @@ class phasecache:
         self._loadedrevslen = 0
         self._phasesets = None
 
-    def phase(self, repo: "localrepo.localrepository", rev: int) -> int:
+    def phase(self, repo: localrepo.localrepository, rev: int) -> int:
         # We need a repo argument here to be able to build _phasesets
         # if necessary. The repository instance is not stored in
         # phasecache to avoid reference cycles. The changelog instance
@@ -872,7 +918,6 @@ class phasecache:
 
         this_phase_set = self._phasesets[targetphase]
         for r in range(start, end):
-
             # gather information about the current_rev
             r_phase = phase(repo, r)
             p_phase = None  # phase inherited from parents
@@ -1011,7 +1056,7 @@ def registernew(repo, tr, targetphase, revs):
     repo._phasecache.replace(phcache)
 
 
-def listphases(repo: "localrepo.localrepository") -> Dict[bytes, bytes]:
+def listphases(repo: localrepo.localrepository) -> Dict[bytes, bytes]:
     """List phases root for serialization over pushkey"""
     # Use ordered dictionary so behavior is deterministic.
     keys = util.sortdict()
@@ -1044,7 +1089,7 @@ def listphases(repo: "localrepo.localrepository") -> Dict[bytes, bytes]:
 
 
 def pushphase(
-    repo: "localrepo.localrepository",
+    repo: localrepo.localrepository,
     nhex: bytes,
     oldphasestr: bytes,
     newphasestr: bytes,
@@ -1095,7 +1140,11 @@ def updatephases(repo, trgetter, headsbyphase):
             advanceboundary(repo, trgetter(), phase, heads)
 
 
-def analyzeremotephases(repo, subset, roots):
+def analyze_remote_phases(
+    repo,
+    subset: Collection[int],
+    roots: Dict[bytes, bytes],
+) -> Tuple[Collection[int], Collection[int]]:
     """Compute phases heads and root in a subset of node from root dict
 
     * subset is heads of the subset
@@ -1105,8 +1154,8 @@ def analyzeremotephases(repo, subset, roots):
     """
     repo = repo.unfiltered()
     # build list from dictionary
-    draftroots = []
-    has_node = repo.changelog.index.has_node  # to filter unknown nodes
+    draft_roots = []
+    to_rev = repo.changelog.index.get_rev
     for nhex, phase in roots.items():
         if nhex == b'publishing':  # ignore data related to publish option
             continue
@@ -1114,49 +1163,53 @@ def analyzeremotephases(repo, subset, roots):
         phase = int(phase)
         if phase == public:
             if node != repo.nullid:
-                repo.ui.warn(
-                    _(
-                        b'ignoring inconsistent public root'
-                        b' from remote: %s\n'
-                    )
-                    % nhex
-                )
+                msg = _(b'ignoring inconsistent public root from remote: %s\n')
+                repo.ui.warn(msg % nhex)
         elif phase == draft:
-            if has_node(node):
-                draftroots.append(node)
+            rev = to_rev(node)
+            if rev is not None:  # to filter unknown nodes
+                draft_roots.append(rev)
         else:
-            repo.ui.warn(
-                _(b'ignoring unexpected root from remote: %i %s\n')
-                % (phase, nhex)
-            )
+            msg = _(b'ignoring unexpected root from remote: %i %s\n')
+            repo.ui.warn(msg % (phase, nhex))
     # compute heads
-    publicheads = newheads(repo, subset, draftroots)
-    return publicheads, draftroots
+    public_heads = new_heads(repo, subset, draft_roots)
+    return public_heads, draft_roots
 
 
-class remotephasessummary:
+class RemotePhasesSummary:
     """summarize phase information on the remote side
 
     :publishing: True is the remote is publishing
-    :publicheads: list of remote public phase heads (nodes)
-    :draftheads: list of remote draft phase heads (nodes)
-    :draftroots: list of remote draft phase root (nodes)
+    :public_heads: list of remote public phase heads (revs)
+    :draft_heads: list of remote draft phase heads (revs)
+    :draft_roots: list of remote draft phase root (revs)
     """
 
-    def __init__(self, repo, remotesubset, remoteroots):
+    def __init__(
+        self,
+        repo,
+        remote_subset: Collection[int],
+        remote_roots: Dict[bytes, bytes],
+    ):
         unfi = repo.unfiltered()
-        self._allremoteroots = remoteroots
+        self._allremoteroots: Dict[bytes, bytes] = remote_roots
 
-        self.publishing = remoteroots.get(b'publishing', False)
+        self.publishing: bool = bool(remote_roots.get(b'publishing', False))
 
-        ana = analyzeremotephases(repo, remotesubset, remoteroots)
-        self.publicheads, self.draftroots = ana
+        heads, roots = analyze_remote_phases(repo, remote_subset, remote_roots)
+        self.public_heads: Collection[int] = heads
+        self.draft_roots: Collection[int] = roots
         # Get the list of all "heads" revs draft on remote
-        dheads = unfi.set(b'heads(%ln::%ln)', self.draftroots, remotesubset)
-        self.draftheads = [c.node() for c in dheads]
+        dheads = unfi.revs(b'heads(%ld::%ld)', roots, remote_subset)
+        self.draft_heads: Collection[int] = dheads
 
 
-def newheads(repo, heads, roots):
+def new_heads(
+    repo,
+    heads: Collection[int],
+    roots: Collection[int],
+) -> Collection[int]:
     """compute new head of a subset minus another
 
     * `heads`: define the first subset
@@ -1165,16 +1218,15 @@ def newheads(repo, heads, roots):
     # phases > dagop > patch > copies > scmutil > obsolete > obsutil > phases
     from . import dagop
 
-    repo = repo.unfiltered()
-    cl = repo.changelog
-    rev = cl.index.get_rev
     if not roots:
         return heads
-    if not heads or heads == [repo.nullid]:
+    if not heads or heads == [nullrev]:
         return []
     # The logic operated on revisions, convert arguments early for convenience
-    new_heads = {rev(n) for n in heads if n != repo.nullid}
-    roots = [rev(n) for n in roots]
+    # PERF-XXX: maybe heads could directly comes as a set without impacting
+    # other user of that value
+    new_heads = set(heads)
+    new_heads.discard(nullrev)
     # compute the area we need to remove
     affected_zone = repo.revs(b"(%ld::%ld)", roots, new_heads)
     # heads in the area are no longer heads
@@ -1192,10 +1244,12 @@ def newheads(repo, heads, roots):
         pruned = dagop.reachableroots(repo, candidates, prunestart)
         new_heads.difference_update(pruned)
 
-    return pycompat.maplist(cl.node, sorted(new_heads))
+    # PERF-XXX: do we actually need a sorted list here? Could we simply return
+    # a set?
+    return sorted(new_heads)
 
 
-def newcommitphase(ui: "uimod.ui") -> int:
+def newcommitphase(ui: uimod.ui) -> int:
     """helper to get the target phase of new commit
 
     Handle all possible values for the phases.new-commit options.
@@ -1210,7 +1264,7 @@ def newcommitphase(ui: "uimod.ui") -> int:
         )
 
 
-def hassecret(repo: "localrepo.localrepository") -> bool:
+def hassecret(repo: localrepo.localrepository) -> bool:
     """utility function that check if a repo have any secret changeset."""
     return bool(repo._phasecache._phaseroots[secret])
 
