@@ -7,10 +7,12 @@
 # GNU General Public License version 2 or any later version.
 """Helper class to compute deltas stored inside revlogs"""
 
+from __future__ import annotations
 
 import abc
 import collections
 import struct
+import typing
 
 # import stuff from node for others to import from revlog
 from ..node import nullrev
@@ -31,6 +33,11 @@ from .constants import (
 
 from ..thirdparty import attr
 
+# Force pytype to use the non-vendored package
+if typing.TYPE_CHECKING:
+    # noinspection PyPackageRequirements
+    import attr
+
 from .. import (
     error,
     mdiff,
@@ -155,8 +162,7 @@ def slicechunk(revlog, revs, targetsize=None):
         revlog.data_config.sr_density_threshold,
         revlog.data_config.sr_min_gap_size,
     ):
-        for subchunk in _slicechunktosize(revlog, chunk, targetsize):
-            yield subchunk
+        yield from _slicechunktosize(revlog, chunk, targetsize)
 
 
 def _slicechunktosize(revlog, revs, targetsize=None):
@@ -428,7 +434,6 @@ def _slicechunktodensity(revlog, revs, targetdensity=0.5, mingapsize=0):
     # Cut the revs at collected indices
     previdx = 0
     for idx in selected:
-
         chunk = _trimchunk(revlog, revs, previdx, idx)
         if chunk:
             yield chunk
