@@ -65,10 +65,17 @@ mappedgenerator
     operation.
 """
 
+from __future__ import annotations
 
 import abc
 import os
 
+from typing import (
+    BinaryIO,
+    Optional,
+    Tuple,
+)
+
 from .i18n import _
 from .pycompat import (
     FileNotFoundError,
@@ -955,12 +962,12 @@ class loader:
                 raise templateutil.TemplateNotFound(
                     _(b'"%s" not in template map') % inst.args[0]
                 )
-            except IOError as inst:
+            except OSError as inst:
                 reason = _(b'template file %s: %s') % (
                     self._map[t],
                     stringutil.forcebytestr(inst.args[1]),
                 )
-                raise IOError(inst.args[0], encoding.strfromlocal(reason))
+                raise OSError(inst.args[0], encoding.strfromlocal(reason))
         return self._parse(self.cache[t])
 
     def _parse(self, tmpl):
@@ -1121,7 +1128,9 @@ def templatedir():
     return path if os.path.isdir(path) else None
 
 
-def open_template(name, templatepath=None):
+def open_template(
+    name: bytes, templatepath: Optional[bytes] = None
+) -> Tuple[bytes, BinaryIO]:
     """returns a file-like object for the given template, and its full path
 
     If the name is a relative path and we're in a frozen binary, the template
@@ -1156,8 +1165,10 @@ def open_template(name, templatepath=None):
     )
 
 
-def try_open_template(name, templatepath=None):
+def try_open_template(
+    name: bytes, templatepath: Optional[bytes] = None
+) -> Tuple[Optional[bytes], Optional[BinaryIO]]:
     try:
         return open_template(name, templatepath)
-    except (EnvironmentError, ImportError):
+    except (OSError, ImportError):
         return None, None
