@@ -7,16 +7,29 @@
 # This software may be used and distributed according to the terms of the
 # GNU General Public License version 2 or any later version.
 
+from __future__ import annotations
 
 import ast
 import codecs
 import re as remod
 import textwrap
 import types
+import typing
+
+from typing import (
+    Iterator,
+    Optional,
+    overload,
+)
 
 from ..i18n import _
 from ..thirdparty import attr
 
+# Force pytype to use the non-vendored package
+if typing.TYPE_CHECKING:
+    # noinspection PyPackageRequirements
+    import attr
+
 from .. import (
     encoding,
     error,
@@ -30,6 +43,16 @@ _regexescapemap = {ord(i): (b'\\' + i).decode('latin1') for i in _respecial}
 regexbytesescapemap = {i: (b'\\' + i) for i in _respecial}
 
 
+@overload
+def reescape(pat: bytes) -> bytes:
+    ...
+
+
+@overload
+def reescape(pat: str) -> str:
+    ...
+
+
 def reescape(pat):
     """Drop-in replacement for re.escape."""
     # NOTE: it is intentional that this works on unicodes and not
@@ -45,12 +68,14 @@ def reescape(pat):
     return pat.encode('latin1')
 
 
-def pprint(o, bprefix=False, indent=0, level=0):
+def pprint(o, bprefix: bool = False, indent: int = 0, level: int = 0) -> bytes:
     """Pretty print an object."""
     return b''.join(pprintgen(o, bprefix=bprefix, indent=indent, level=level))
 
 
-def pprintgen(o, bprefix=False, indent=0, level=0):
+def pprintgen(
+    o, bprefix: bool = False, indent: int = 0, level: int = 0
+) -> Iterator[bytes]:
     """Pretty print an object to a generator of atoms.
 
     ``bprefix`` is a flag influencing whether bytestrings are preferred with
@@ -85,10 +110,7 @@ def pprintgen(o, bprefix=False, indent=0, level=0):
             yield b' ' * (level * indent)
 
         for i, a in enumerate(o):
-            for chunk in pprintgen(
-                a, bprefix=bprefix, indent=indent, level=level
-            ):
-                yield chunk
+            yield from pprintgen(a, bprefix=bprefix, indent=indent, level=level)
 
             if i + 1 < len(o):
                 if indent:
@@ -116,17 +138,11 @@ def pprintgen(o, bprefix=False, indent=0, level=0):
             yield b' ' * (level * indent)
 
         for i, (k, v) in enumerate(sorted(o.items())):
-            for chunk in pprintgen(
-                k, bprefix=bprefix, indent=indent, level=level
-            ):
-                yield chunk
+            yield from pprintgen(k, bprefix=bprefix, indent=indent, level=level)
 
             yield b': '
 
-            for chunk in pprintgen(
-                v, bprefix=bprefix, indent=indent, level=level
-            ):
-                yield chunk
+            yield from pprintgen(v, bprefix=bprefix, indent=indent, level=level)
 
             if i + 1 < len(o):
                 if indent:
@@ -154,10 +170,7 @@ def pprintgen(o, bprefix=False, indent=0, level=0):
             yield b' ' * (level * indent)
 
         for i, k in enumerate(sorted(o)):
-            for chunk in pprintgen(
-                k, bprefix=bprefix, indent=indent, level=level
-            ):
-                yield chunk
+            yield from pprintgen(k, bprefix=bprefix, indent=indent, level=level)
 
             if i + 1 < len(o):
                 if indent:
@@ -185,10 +198,7 @@ def pprintgen(o, bprefix=False, indent=0, level=0):
             yield b' ' * (level * indent)
 
         for i, a in enumerate(o):
-            for chunk in pprintgen(
-                a, bprefix=bprefix, indent=indent, level=level
-            ):
-                yield chunk
+            yield from pprintgen(a, bprefix=bprefix, indent=indent, level=level)
 
             if i + 1 < len(o):
                 if indent:
@@ -228,10 +238,9 @@ def pprintgen(o, bprefix=False, indent=0, level=0):
             except StopIteration:
                 last = True
 
-            for chunk in pprintgen(
+            yield from pprintgen(
                 current, bprefix=bprefix, indent=indent, level=level
-            ):
-                yield chunk
+            )
 
             if not last:
                 if indent:
@@ -250,7 +259,7 @@ def pprintgen(o, bprefix=False, indent=0, level=0):
         yield pycompat.byterepr(o)
 
 
-def prettyrepr(o):
+def prettyrepr(o) -> bytes:
     """Pretty print a representation of a possibly-nested object"""
     lines = []
     rs = pycompat.byterepr(o)
@@ -281,7 +290,7 @@ def prettyrepr(o):
     return b'\n'.join(b'  ' * l + s for l, s in lines)
 
 
-def buildrepr(r):
+def buildrepr(r) -> bytes:
     """Format an optional printable representation from unexpanded bits
 
     ========  =================================
@@ -305,12 +314,12 @@ def buildrepr(r):
         return pprint(r)
 
 
-def binary(s):
+def binary(s: bytes) -> bool:
     """return true if a string is binary data"""
     return bool(s and b'\0' in s)
 
 
-def _splitpattern(pattern):
+def _splitpattern(pattern: bytes):
     if pattern.startswith(b're:'):
         return b're', pattern[3:]
     elif pattern.startswith(b'literal:'):
@@ -318,7 +327,7 @@ def _splitpattern(pattern):
     return b'literal', pattern
 
 
-def stringmatcher(pattern, casesensitive=True):
+def stringmatcher(pattern: bytes, casesensitive: bool = True):
     """
     accepts a string, possibly starting with 're:' or 'literal:' prefix.
     returns the matcher name, pattern, and matcher function.
@@ -379,7 +388,7 @@ def stringmatcher(pattern, casesensitive=True):
     raise error.ProgrammingError(b'unhandled pattern kind: %s' % kind)
 
 
-def substringregexp(pattern, flags=0):
+def substringregexp(pattern: bytes, flags: int = 0):
     """Build a regexp object from a string pattern possibly starting with
     're:' or 'literal:' prefix.
 
@@ -431,7 +440,7 @@ def substringregexp(pattern, flags=0):
     raise error.ProgrammingError(b'unhandled pattern kind: %s' % kind)
 
 
-def shortuser(user):
+def shortuser(user: bytes) -> bytes:
     """Return a short representation of a user name or email address."""
     f = user.find(b'@')
     if f >= 0:
@@ -448,7 +457,7 @@ def shortuser(user):
     return user
 
 
-def emailuser(user):
+def emailuser(user: bytes) -> bytes:
     """Return the user portion of an email address."""
     f = user.find(b'@')
     if f >= 0:
@@ -459,7 +468,7 @@ def emailuser(user):
     return user
 
 
-def email(author):
+def email(author: bytes) -> bytes:
     '''get email of author.'''
     r = author.find(b'>')
     if r == -1:
@@ -467,7 +476,7 @@ def email(author):
     return author[author.find(b'<') + 1 : r]
 
 
-def person(author):
+def person(author: bytes) -> bytes:
     """Returns the name before an email address,
     interpreting it as per RFC 5322
 
@@ -559,7 +568,6 @@ def parsemailmap(mailmapcontent):
         return mailmap
 
     for line in mailmapcontent.splitlines():
-
         # Don't bother checking the line if it is a comment or
         # is an improperly formed author field
         if line.lstrip().startswith(b'#'):
@@ -612,7 +620,7 @@ def parsemailmap(mailmapcontent):
     return mailmap
 
 
-def mapname(mailmap, author):
+def mapname(mailmap, author: bytes) -> bytes:
     """Returns the author field according to the mailmap cache, or
     the original author field.
 
@@ -663,7 +671,7 @@ def mapname(mailmap, author):
 _correctauthorformat = remod.compile(br'^[^<]+\s<[^<>]+@[^<>]+>$')
 
 
-def isauthorwellformed(author):
+def isauthorwellformed(author: bytes) -> bool:
     """Return True if the author field is well formed
     (ie "Contributor Name <contrib@email.dom>")
 
@@ -685,7 +693,7 @@ def isauthorwellformed(author):
     return _correctauthorformat.match(author) is not None
 
 
-def firstline(text):
+def firstline(text: bytes) -> bytes:
     """Return the first line of the input"""
     # Try to avoid running splitlines() on the whole string
     i = text.find(b'\n')
@@ -697,21 +705,26 @@ def firstline(text):
         return b''
 
 
-def ellipsis(text, maxlength=400):
+def ellipsis(text: bytes, maxlength: int = 400) -> bytes:
     """Trim string to at most maxlength (default: 400) columns in display."""
     return encoding.trim(text, maxlength, ellipsis=b'...')
 
 
-def escapestr(s):
-    if isinstance(s, memoryview):
+def escapestr(s: bytes) -> bytes:
+    # "bytes" is also a typing shortcut for bytes, bytearray, and memoryview
+    if isinstance(s, (memoryview, bytearray)):
         s = bytes(s)
     # call underlying function of s.encode('string_escape') directly for
     # Python 3 compatibility
+    # pytype: disable=bad-return-type
     return codecs.escape_encode(s)[0]  # pytype: disable=module-attr
+    # pytype: enable=bad-return-type
 
 
-def unescapestr(s):
+def unescapestr(s: bytes) -> bytes:
+    # pytype: disable=bad-return-type
     return codecs.escape_decode(s)[0]  # pytype: disable=module-attr
+    # pytype: enable=bad-return-type
 
 
 def forcebytestr(obj):
@@ -724,7 +737,7 @@ def forcebytestr(obj):
         return pycompat.bytestr(encoding.strtolocal(str(obj)))
 
 
-def uirepr(s):
+def uirepr(s: bytes) -> bytes:
     # Avoid double backslash in Windows path repr()
     return pycompat.byterepr(pycompat.bytestr(s)).replace(b'\\\\', b'\\')
 
@@ -781,7 +794,6 @@ def _MBTextWrapper(**kwargs):
             chunks.reverse()
 
             while chunks:
-
                 # Start the list of chunks that will make up the current line.
                 # cur_len is just the length of all the chunks in cur_line.
                 cur_line = []
@@ -838,7 +850,9 @@ def _MBTextWrapper(**kwargs):
     return tw(**kwargs)
 
 
-def wrap(line, width, initindent=b'', hangindent=b''):
+def wrap(
+    line: bytes, width: int, initindent: bytes = b'', hangindent: bytes = b''
+) -> bytes:
     maxindent = max(len(hangindent), len(initindent))
     if width <= maxindent:
         # adjust for weird terminal size
@@ -875,7 +889,7 @@ _booleans = {
 }
 
 
-def parsebool(s):
+def parsebool(s: bytes) -> Optional[bool]:
     """Parse s into a boolean.
 
     If s is not a valid boolean, returns None.
@@ -883,7 +897,8 @@ def parsebool(s):
     return _booleans.get(s.lower(), None)
 
 
-def parselist(value):
+# TODO: make arg mandatory (and fix code below?)
+def parselist(value: Optional[bytes]):
     """parse a configuration value as a list of comma/space separated strings
 
     >>> parselist(b'this,is "a small" ,test')
@@ -973,7 +988,7 @@ def parselist(value):
     return result or []
 
 
-def evalpythonliteral(s):
+def evalpythonliteral(s: bytes):
     """Evaluate a string containing a Python literal expression"""
     # We could backport our tokenizer hack to rewrite '' to u'' if we want
     return ast.literal_eval(s.decode('latin1'))
