@@ -5,6 +5,7 @@
 # This software may be used and distributed according to the terms of the
 # GNU General Public License version 2 or any later version.
 
+from __future__ import annotations
 
 import errno
 import getpass
@@ -190,9 +191,9 @@ def posixfile(name, mode=b'r', buffering=-1):
             return mixedfilemodewrapper(fp)
 
         return fp
-    except WindowsError as err:  # pytype: disable=name-error
+    except OSError as err:
         # convert to a friendlier exception
-        raise IOError(
+        raise OSError(
             err.errno, '%s: %s' % (encoding.strfromlocal(name), err.strerror)
         )
 
@@ -207,19 +208,19 @@ def get_password() -> bytes:
     This shouldn't be called directly- use ``ui.getpass()`` instead, which
     checks if the session is interactive first.
     """
-    pw = u""
+    pw = ""
     while True:
         c = msvcrt.getwch()  # pytype: disable=module-attr
-        if c == u'\r' or c == u'\n':
+        if c == '\r' or c == '\n':
             break
-        if c == u'\003':
+        if c == '\003':
             raise KeyboardInterrupt
-        if c == u'\b':
+        if c == '\b':
             pw = pw[:-1]
         else:
             pw = pw + c
-    msvcrt.putwch(u'\r')  # pytype: disable=module-attr
-    msvcrt.putwch(u'\n')  # pytype: disable=module-attr
+    msvcrt.putwch('\r')  # pytype: disable=module-attr
+    msvcrt.putwch('\n')  # pytype: disable=module-attr
     return encoding.unitolocal(pw)
 
 
@@ -241,25 +242,25 @@ class winstdout(typelib.BinaryIO_Proxy):
     def close(self):
         try:
             self.fp.close()
-        except IOError:
+        except OSError:
             pass
 
     def write(self, s):
         try:
             return self.fp.write(s)
-        except IOError as inst:
+        except OSError as inst:
             if inst.errno != 0 and not win32.lasterrorwaspipeerror(inst):
                 raise
             self.close()
-            raise IOError(errno.EPIPE, 'Broken pipe')
+            raise OSError(errno.EPIPE, 'Broken pipe')
 
     def flush(self):
         try:
             return self.fp.flush()
-        except IOError as inst:
+        except OSError as inst:
             if not win32.lasterrorwaspipeerror(inst):
                 raise
-            raise IOError(errno.EPIPE, 'Broken pipe')
+            raise OSError(errno.EPIPE, 'Broken pipe')
 
 
 def openhardlinks() -> bool:
@@ -298,7 +299,7 @@ def setflags(f: bytes, l: bool, x: bool) -> None:
 def copymode(
     src: bytes,
     dst: bytes,
-    mode: Optional[bytes] = None,
+    mode: Optional[int] = None,
     enforcewritable: bool = False,
 ) -> None:
     pass
@@ -621,10 +622,10 @@ def groupname(gid: Optional[int] = None) -> Optional[bytes]:
     return None
 
 
-def readlink(pathname: bytes) -> bytes:
-    path = pycompat.fsdecode(pathname)
+def readlink(path: bytes) -> bytes:
+    path_str = pycompat.fsdecode(path)
     try:
-        link = os.readlink(path)
+        link = os.readlink(path_str)
     except ValueError as e:
         # On py2, os.readlink() raises an AttributeError since it is
         # unsupported.  On py3, reading a non-link raises a ValueError.  Simply
@@ -743,7 +744,7 @@ def lookupreg(
 
                 # never let a Unicode string escape into the wild
                 return encoding.unitolocal(val)
-        except EnvironmentError:
+        except OSError:
             pass
 
 
