@@ -5,7 +5,10 @@ use format_bytes::{format_bytes, write_bytes, DisplayBytes};
 use crate::{
     errors::HgError,
     exit_codes::STATE_ERROR,
-    filepatterns::parse_pattern_file_contents,
+    filepatterns::{
+        parse_pattern_file_contents, IgnorePattern, PatternError,
+        PatternFileWarning, PatternSyntax,
+    },
     matchers::{
         AlwaysMatcher, DifferenceMatcher, IncludeMatcher, Matcher,
         UnionMatcher,
@@ -14,9 +17,8 @@ use crate::{
     operations::cat,
     repo::Repo,
     requirements::SPARSE_REQUIREMENT,
-    utils::{hg_path::HgPath, SliceExt},
-    IgnorePattern, PatternError, PatternFileWarning, PatternSyntax, Revision,
-    NULL_REVISION,
+    utils::{hg_path::HgPath, strings::SliceExt},
+    Revision, NULL_REVISION,
 };
 
 /// Command which is triggering the config read
