@@ -11,12 +11,11 @@
 use cpython::{
     PyBytes, PyClone, PyDict, PyObject, PyResult, Python, UnsafePyLeaked,
 };
-use std::cell::RefCell;
 
 use crate::dirstate::dirstate_map::v2_error;
 use crate::dirstate::dirstate_map::DirstateMap;
+use hg::dirstate::on_disk::DirstateV2ParseError;
 use hg::dirstate::CopyMapIter;
-use hg::dirstate_tree::on_disk::DirstateV2ParseError;
 use hg::utils::hg_path::HgPath;
 
 py_class!(pub class CopyMap |py| {
