@@ -33,6 +33,7 @@
 import os
 import sys
 
+
 # Generates pairs of (filename, contents), where 'contents' is a list
 # describing the file's content at each revision (or in the working copy).
 # At each revision, it is either None or the file's actual content. When not
@@ -57,10 +58,7 @@ def generatestates(maxchangesets, parentcontents):
         for content in {None, b'content' + (b"%d" % (depth + 1))} | set(
             parentcontents
         ):
-            for combination in generatestates(
-                maxchangesets, parentcontents + [content]
-            ):
-                yield combination
+            yield from generatestates(maxchangesets, parentcontents + [content])
 
 
 # retrieve the command line arguments
