@@ -43,7 +43,17 @@ def testmod(name, optionflags=0, testtarget=None):
     checker = py3docchecker()
     runner = doctest.DocTestRunner(checker=checker, optionflags=optionflags)
     for test in finder.find(mod, name):
-        runner.run(test)
+        # Windows doesn't have time.tzset(), so skip methods that invoke it in
+        # a doctest, without hardcoding the function name.  There is a feature
+        # request for adding syntax to the test itself to conditionally skip
+        # that would make this unnecessary:
+        #
+        # https://github.com/python/cpython/issues/117364
+        for example in test.examples:
+            if os.name == 'nt' and 'time.tzset()' in example.source:
+                break
+        else:
+            runner.run(test)
     runner.summarize()
 
 
@@ -71,14 +81,14 @@ cwd = os.path.dirname(os.environ["TESTDIR"])
 if not os.path.isdir(os.path.join(cwd, ".hg")):
     sys.exit(0)
 
-files_cmd = "hg files --print0 \"%s\"" % fileset
-
-if 'HGTEST_RESTOREENV' in os.environ:
-    files_cmd = '. $HGTEST_RESTOREENV; ' + files_cmd
-
 files = subprocess.check_output(
-    files_cmd,
-    shell=True,
+    [
+        "sh",
+        "-c",
+        "set -e; . helpers-testrepo.sh; testrepohg files --print0 \"%s\""
+        % fileset,
+    ],
+    shell=False,
     cwd=cwd,
 ).split(b'\0')
 
@@ -119,51 +129,49 @@ for f in files:
 # Meta-test: let's make sure that we actually ran what we expected to, above.
 # Each item in the set is a 2-tuple of module name and stringified kwargs passed
 # to testmod.
-expected_mods_tested = set(
-    [
-        ('hgext.convert.convcmd', '{}'),
-        ('hgext.convert.cvsps', '{}'),
-        ('hgext.convert.filemap', '{}'),
-        ('hgext.convert.p4', '{}'),
-        ('hgext.convert.subversion', '{}'),
-        ('hgext.fix', '{}'),
-        ('hgext.mq', '{}'),
-        ('mercurial.changelog', '{}'),
-        ('mercurial.cmdutil', '{}'),
-        ('mercurial.color', '{}'),
-        ('mercurial.dagparser', "{'optionflags': 4}"),
-        ('mercurial.dirstateutils.v2', '{}'),
-        ('mercurial.encoding', '{}'),
-        ('mercurial.fancyopts', '{}'),
-        ('mercurial.formatter', '{}'),
-        ('mercurial.hg', '{}'),
-        ('mercurial.hgweb.hgwebdir_mod', '{}'),
-        ('mercurial.match', '{}'),
-        ('mercurial.mdiff', '{}'),
-        ('mercurial.minirst', '{}'),
-        ('mercurial.parser', '{}'),
-        ('mercurial.patch', '{}'),
-        ('mercurial.pathutil', '{}'),
-        ('mercurial.pycompat', '{}'),
-        ('mercurial.revlogutils.deltas', '{}'),
-        ('mercurial.revset', '{}'),
-        ('mercurial.revsetlang', '{}'),
-        ('mercurial.simplemerge', '{}'),
-        ('mercurial.smartset', '{}'),
-        ('mercurial.store', '{}'),
-        ('mercurial.subrepo', '{}'),
-        ('mercurial.templater', '{}'),
-        ('mercurial.ui', '{}'),
-        ('mercurial.util', "{'testtarget': 'platform'}"),
-        ('mercurial.util', '{}'),
-        ('mercurial.utils.dateutil', '{}'),
-        ('mercurial.utils.stringutil', '{}'),
-        ('mercurial.utils.urlutil', '{}'),
-        ('tests.drawdag', '{}'),
-        ('tests.test-run-tests', '{}'),
-        ('tests.test-url', "{'optionflags': 4}"),
-    ]
-)
+expected_mods_tested = {
+    ('hgext.convert.convcmd', '{}'),
+    ('hgext.convert.cvsps', '{}'),
+    ('hgext.convert.filemap', '{}'),
+    ('hgext.convert.p4', '{}'),
+    ('hgext.convert.subversion', '{}'),
+    ('hgext.fix', '{}'),
+    ('hgext.mq', '{}'),
+    ('mercurial.changelog', '{}'),
+    ('mercurial.cmdutil', '{}'),
+    ('mercurial.color', '{}'),
+    ('mercurial.dagparser', "{'optionflags': 4}"),
+    ('mercurial.dirstateutils.v2', '{}'),
+    ('mercurial.encoding', '{}'),
+    ('mercurial.fancyopts', '{}'),
+    ('mercurial.formatter', '{}'),
+    ('mercurial.hg', '{}'),
+    ('mercurial.hgweb.hgwebdir_mod', '{}'),
+    ('mercurial.match', '{}'),
+    ('mercurial.mdiff', '{}'),
+    ('mercurial.minirst', '{}'),
+    ('mercurial.parser', '{}'),
+    ('mercurial.patch', '{}'),
+    ('mercurial.pathutil', '{}'),
+    ('mercurial.pycompat', '{}'),
+    ('mercurial.revlogutils.deltas', '{}'),
+    ('mercurial.revset', '{}'),
+    ('mercurial.revsetlang', '{}'),
+    ('mercurial.simplemerge', '{}'),
+    ('mercurial.smartset', '{}'),
+    ('mercurial.store', '{}'),
+    ('mercurial.subrepo', '{}'),
+    ('mercurial.templater', '{}'),
+    ('mercurial.ui', '{}'),
+    ('mercurial.util', "{'testtarget': 'platform'}"),
+    ('mercurial.util', '{}'),
+    ('mercurial.utils.dateutil', '{}'),
+    ('mercurial.utils.stringutil', '{}'),
+    ('mercurial.utils.urlutil', '{}'),
+    ('tests.drawdag', '{}'),
+    ('tests.test-run-tests', '{}'),
+    ('tests.test-url', "{'optionflags': 4}"),
+}
 
 unexpectedly_run = mods_tested.difference(expected_mods_tested)
 not_run = expected_mods_tested.difference(mods_tested)
